"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAnnouncedVisitor = exports.combineVisitors = exports.announceDelta = exports.applyDelta = exports.makeDetachedFieldIndex = void 0;
const index_js_1 = require("../../util/index.js");
const detachedFieldIndex_js_1 = require("./detachedFieldIndex.js");
const visitDelta_js_1 = require("./visitDelta.js");
function makeDetachedFieldIndex(prefix = "Temp", revisionTagCodec, idCompressor, options) {
    return new detachedFieldIndex_js_1.DetachedFieldIndex(prefix, (0, index_js_1.idAllocatorFromMaxId)(), revisionTagCodec, idCompressor, options);
}
exports.makeDetachedFieldIndex = makeDetachedFieldIndex;
function applyDelta(delta, latestRevision, deltaProcessor, detachedFieldIndex) {
    const visitor = deltaProcessor.acquireVisitor();
    (0, visitDelta_js_1.visitDelta)(delta, visitor, detachedFieldIndex, latestRevision);
    visitor.free();
}
exports.applyDelta = applyDelta;
function announceDelta(delta, latestRevision, deltaProcessor, detachedFieldIndex) {
    const visitor = deltaProcessor.acquireVisitor();
    (0, visitDelta_js_1.visitDelta)(delta, combineVisitors([visitor]), detachedFieldIndex, latestRevision);
    visitor.free();
}
exports.announceDelta = announceDelta;
/**
 * Combines multiple visitors into a single visitor.
 * @param visitors - The returned visitor invokes the corresponding events for all these visitors, in order.
 * @returns a DeltaVisitor combining all `visitors`.
 */
function combineVisitors(visitors) {
    const allVisitors = visitors.flatMap((v) => (v.type === "Combined" ? v.visitors : [v]));
    const announcedVisitors = allVisitors.filter((v) => v.type === "Announced");
    return {
        type: "Combined",
        visitors: allVisitors,
        free: () => visitors.forEach((v) => v.free()),
        create: (...args) => {
            allVisitors.forEach((v) => v.create(...args));
            announcedVisitors.forEach((v) => v.afterCreate(...args));
        },
        destroy: (...args) => {
            announcedVisitors.forEach((v) => v.beforeDestroy(...args));
            allVisitors.forEach((v) => v.destroy(...args));
        },
        attach: (source, count, destination) => {
            announcedVisitors.forEach((v) => v.beforeAttach(source, count, destination));
            allVisitors.forEach((v) => v.attach(source, count, destination));
            announcedVisitors.forEach((v) => v.afterAttach(source, {
                start: destination,
                end: destination + count,
            }));
        },
        detach: (source, destination, id, isReplaced) => {
            announcedVisitors.forEach((v) => v.beforeDetach(source, destination, isReplaced));
            allVisitors.forEach((v) => v.detach(source, destination, id, isReplaced));
            announcedVisitors.forEach((v) => v.afterDetach(source.start, source.end - source.start, destination, isReplaced));
        },
        enterNode: (...args) => allVisitors.forEach((v) => v.enterNode(...args)),
        exitNode: (...args) => allVisitors.forEach((v) => v.exitNode(...args)),
        enterField: (...args) => allVisitors.forEach((v) => v.enterField(...args)),
        exitField: (...args) => allVisitors.forEach((v) => v.exitField(...args)),
    };
}
exports.combineVisitors = combineVisitors;
/**
 * Creates an announced visitor with only the provided functions and uses a no op for the rest.
 * This is provided to make some of the delta visitor definitions cleaner.
 */
function createAnnouncedVisitor(visitorFunctions) {
    const noOp = () => { };
    return {
        type: "Announced",
        free: visitorFunctions.free ?? noOp,
        create: visitorFunctions.create ?? noOp,
        afterCreate: visitorFunctions.afterCreate ?? noOp,
        beforeDestroy: visitorFunctions.beforeDestroy ?? noOp,
        destroy: visitorFunctions.destroy ?? noOp,
        beforeAttach: visitorFunctions.beforeAttach ?? noOp,
        attach: visitorFunctions.attach ?? noOp,
        afterAttach: visitorFunctions.afterAttach ?? noOp,
        beforeDetach: visitorFunctions.beforeDetach ?? noOp,
        detach: visitorFunctions.detach ?? noOp,
        afterDetach: visitorFunctions.afterDetach ?? noOp,
        enterNode: visitorFunctions.enterNode ?? noOp,
        exitNode: visitorFunctions.exitNode ?? noOp,
        enterField: visitorFunctions.enterField ?? noOp,
        exitField: visitorFunctions.exitField ?? noOp,
    };
}
exports.createAnnouncedVisitor = createAnnouncedVisitor;
//# sourceMappingURL=visitorUtils.js.map