/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { type ImplicitAllowedTypes, type InsertableTreeNodeFromImplicitAllowedTypes, type NodeKind, type SchemaFactoryAlpha, type ScopedSchemaName, TreeArrayNode, type TreeNode, type TreeNodeFromImplicitAllowedTypes, type TreeNodeSchemaClass, type WithType } from "./simple-tree/index.js";
/**
 * Contains types and factories for creating schema to represent dynamic tabular data.
 * @privateRemarks TODO: document in more detail and add `@example`s.
 * @internal
 */
export declare namespace TableSchema {
    /**
     * A column in a table.
     * @remarks Implemented by the schema class returned from {@link TableSchema.createColumn}.
     * @sealed @internal
     */
    interface IColumn<TPropsSchema extends ImplicitAllowedTypes = ImplicitAllowedTypes> {
        /**
         * The unique identifier of the column.
         * @remarks Uniquely identifies the node within the entire tree, not just the table.
         */
        readonly id: string;
        /**
         * User-provided column properties.
         */
        get props(): TreeNodeFromImplicitAllowedTypes<TPropsSchema>;
        set props(value: InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema>);
    }
    /**
     * Factory for creating new table column schema.
     * @privateRemarks
     * TODO:
     * - Add overloads to make propsSchema optional.
     * - Take field schema rather than node schema for `propsSchema`, in particular to allow making
     * the additional properties optional.
     * @internal
     */
    function createColumn<const TInputScope extends string | undefined, const TPropsSchema extends ImplicitAllowedTypes>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, propsSchema: TPropsSchema): TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Column">, NodeKind.Object, TreeNode & IColumn<TPropsSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Column">, NodeKind, unknown>, object & {
        readonly id?: string | undefined;
        readonly props: InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema> | (undefined & InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema>);
    }, true, {
        readonly id: import("./simple-tree/schemaTypes.js").FieldSchema<import("./simple-tree/schemaTypes.js").FieldKind.Identifier, import("./simple-tree/leafNodeSchema.js").LeafSchema<"string", string>, unknown>;
        readonly props: import("./simple-tree/schemaTypes.js").FieldSchema<import("./simple-tree/schemaTypes.js").FieldKind.Required, TPropsSchema, unknown>;
    }>;
    /**
     * Base column schema type.
     * @sealed @system @internal
     */
    type ColumnSchemaBase<TScope extends string | undefined, TPropsSchema extends ImplicitAllowedTypes> = ReturnType<typeof createColumn<TScope, TPropsSchema>>;
    /**
     * A row in a table.
     * @remarks Implemented by the schema class returned from {@link TableSchema.createRow}.
     * @sealed @internal
     */
    interface IRow<TCellSchema extends ImplicitAllowedTypes, TPropsSchema extends ImplicitAllowedTypes = ImplicitAllowedTypes> {
        /**
         * The unique identifier of the row.
         * @remarks Uniquely identifies the node within the entire tree, not just the table.
         */
        readonly id: string;
        /**
         * Gets the cell in the specified column.
         * @returns The cell if it exists, otherwise undefined.
         */
        getCell(column: IColumn): TreeNodeFromImplicitAllowedTypes<TCellSchema> | undefined;
        /**
         * Gets the cell in the specified column, denoted by column ID.
         * @returns The cell if it exists, otherwise undefined.
         */
        getCell(columnId: string): TreeNodeFromImplicitAllowedTypes<TCellSchema> | undefined;
        /**
         * Sets the cell in the specified column.
         * @remarks To remove a cell, call {@link TableSchema.IRow.(removeCell:1)} instead.
         */
        setCell(column: IColumn, value: InsertableTreeNodeFromImplicitAllowedTypes<TCellSchema>): void;
        /**
         * Sets the cell in the specified column, denoted by column ID.
         * @remarks To remove a cell, call {@link TableSchema.IRow.(removeCell:2)} instead.
         */
        setCell(columnId: string, value: InsertableTreeNodeFromImplicitAllowedTypes<TCellSchema>): void;
        /**
         * Removes the cell in the specified column.
         * @privateRemarks TODO: return removed cell
         */
        removeCell(column: IColumn): void;
        /**
         * Removes the cell in the specified column, denoted by column ID.
         * @privateRemarks TODO: return removed cell
         */
        removeCell(columnId: string): void;
        /**
         * User-provided row properties.
         */
        get props(): TreeNodeFromImplicitAllowedTypes<TPropsSchema>;
        set props(value: InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema>);
    }
    /**
     * Factory for creating new table row schema.
     *
     * @privateRemarks
     * TODO:
     * - Add overloads to make propsSchema optional.
     * - Take field schema rather than node schema for `propsSchema`, in particular to allow making
     * the additional properties optional.
     *
     * @sealed @internal
     */
    function createRow<const TInputScope extends string | undefined, const TCellSchema extends ImplicitAllowedTypes, const TPropsSchema extends ImplicitAllowedTypes>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, cellSchema: TCellSchema, propsSchema: TPropsSchema): TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row">, NodeKind.Object, TreeNode & IRow<TCellSchema, TPropsSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row">, NodeKind, unknown>, object & {
        readonly id?: string | undefined;
        readonly cells: (import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, true, TCellSchema, import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema> | undefined) => import("./simple-tree/mapNode.js").TreeMapNode<TCellSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, true, TCellSchema, import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema> | undefined) => import("./simple-tree/mapNode.js").TreeMapNode<TCellSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, unknown>)> | undefined) & import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, true, TCellSchema, import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema> | undefined) => import("./simple-tree/mapNode.js").TreeMapNode<TCellSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, true, TCellSchema, import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema> | undefined) => import("./simple-tree/mapNode.js").TreeMapNode<TCellSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, unknown>)>;
        readonly props: InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema> | (undefined & InsertableTreeNodeFromImplicitAllowedTypes<TPropsSchema>);
    }, true, {
        readonly id: import("./simple-tree/schemaTypes.js").FieldSchema<import("./simple-tree/schemaTypes.js").FieldKind.Identifier, import("./simple-tree/leafNodeSchema.js").LeafSchema<"string", string>, unknown>;
        readonly cells: TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, import("./simple-tree/mapNode.js").TreeMapNode<TCellSchema> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Row.cells">, NodeKind.Map, unknown>, import("./simple-tree/mapNode.js").MapNodeInsertableData<TCellSchema>, true, TCellSchema, undefined>;
        readonly props: import("./simple-tree/schemaTypes.js").FieldSchema<import("./simple-tree/schemaTypes.js").FieldKind.Required, TPropsSchema, unknown>;
    }>;
    /**
     * Base row schema type.
     * @sealed @system @internal
     */
    type RowSchemaBase<TScope extends string | undefined, TCellSchema extends ImplicitAllowedTypes, TPropsSchema extends ImplicitAllowedTypes> = ReturnType<typeof createRow<TScope, TCellSchema, TPropsSchema>>;
    /**
     * A key to uniquely identify a cell in a table.
     * @sealed @internal
     */
    interface CellKey {
        /**
         * {@link TableSchema.IColumn.id} of the containing {@link TableSchema.IColumn}.
         */
        readonly columnId: string;
        /**
         * {@link TableSchema.IRow.id} of the containing {@link TableSchema.IRow}.
         */
        readonly rowId: string;
    }
    /**
     * {@link TableSchema.ITable.insertColumn} parameters.
     * @sealed @internal
     */
    interface InsertColumnParameters<TInsertableColumn> {
        /**
         * The index at which to insert the new column.
         * @remarks If not provided, the column will be appended to the end of the table.
         */
        readonly index?: number | undefined;
        /**
         * The column to insert.
         */
        readonly column: TInsertableColumn;
    }
    /**
     * {@link TableSchema.ITable.insertRows} parameters.
     * @sealed @internal
     */
    interface InsertRowsParameters<TInsertableRow> {
        /**
         * The index at which to insert the new rows.
         * @remarks If not provided, the rows will be appended to the end of the table.
         */
        readonly index?: number | undefined;
        /**
         * The rows to insert.
         */
        readonly rows: TInsertableRow[];
    }
    /**
     * {@link TableSchema.ITable.setCell} parameters.
     * @sealed @internal
     */
    interface SetCellParameters<TInsertableCell> {
        /**
         * The key to uniquely identify a cell in a table.
         */
        readonly key: CellKey;
        /**
         * The cell to set.
         */
        readonly cell: TInsertableCell;
    }
    /**
     * A table.
     * @sealed @internal
     */
    interface ITable<TCellSchema extends ImplicitAllowedTypes, TColumnSchema extends ImplicitAllowedTypes, TRowSchema extends ImplicitAllowedTypes> {
        /**
         * The table's columns.
         */
        readonly columns: TreeArrayNode<TColumnSchema>;
        /**
         * The table's rows.
         */
        readonly rows: TreeArrayNode<TRowSchema>;
        /**
         * Gets a table column by its {@link TableSchema.IRow.id}.
         */
        getColumn(id: string): TreeNodeFromImplicitAllowedTypes<TColumnSchema> | undefined;
        /**
         * Gets a table row by its {@link TableSchema.IRow.id}.
         */
        getRow(id: string): TreeNodeFromImplicitAllowedTypes<TRowSchema> | undefined;
        /**
         * Gets a cell in the table by column and row IDs.
         * @param key - A key that uniquely distinguishes a cell in the table, represented as a combination of the column ID and row ID.
         * @privateRemarks TODO: add overload that takes row and column nodes.
         */
        getCell(key: CellKey): TreeNodeFromImplicitAllowedTypes<TCellSchema> | undefined;
        /**
         * Inserts a column into the table.
         * @throws Throws an error if the column is already in the tree, or if the specified index is out of range.
         */
        insertColumn(params: InsertColumnParameters<InsertableTreeNodeFromImplicitAllowedTypes<TColumnSchema>>): TreeNodeFromImplicitAllowedTypes<TColumnSchema>;
        /**
         * Inserts 0 or more rows into the table.
         * @throws Throws an error if any of the rows are already in the tree, or if the specified index is out of range.
         */
        insertRows(params: InsertRowsParameters<InsertableTreeNodeFromImplicitAllowedTypes<TRowSchema>>): TreeNodeFromImplicitAllowedTypes<TRowSchema>[];
        /**
         * Sets the cell at the specified location in the table.
         * @remarks To remove a cell, call {@link TableSchema.ITable.removeCell} instead.
         * @privateRemarks TODO: add overload that takes column/row nodes?
         */
        setCell(params: SetCellParameters<InsertableTreeNodeFromImplicitAllowedTypes<TCellSchema>>): void;
        /**
         * Removes the specified column from the table.
         * @remarks Note: this does not remove any cells from the table's rows.
         * @privateRemarks
         * TODO:
         * - Policy for when the column is not in the table.
         * - Actually remove corresponding cells from table rows.
         */
        removeColumn: (column: TreeNodeFromImplicitAllowedTypes<TColumnSchema>) => void;
        /**
         * Removes 0 or more rows from the table.
         * @privateRemarks TODO: policy for when 1 or more rows are not in the table.
         */
        removeRows: (rows: readonly TreeNodeFromImplicitAllowedTypes<TRowSchema>[]) => void;
        /**
         * Removes all rows from the table.
         */
        removeAllRows: () => void;
        /**
         * Removes the cell at the specified location in the table.
         * @privateRemarks TODO: add overload that takes column/row nodes?
         */
        removeCell: (key: CellKey) => void;
    }
    /**
     * Factory for creating new table schema without specifying row or column schema.
     * @internal
     */
    function createTable<const TInputScope extends string | undefined, const TCell extends ImplicitAllowedTypes>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, _cellSchema: TCell): ReturnType<typeof createTableInternal<TInputScope, TCell>>;
    /**
     * Factory for creating new table schema without specifying row schema
     * @internal
     */
    function createTable<const TInputScope extends string | undefined, const TCell extends ImplicitAllowedTypes, const TColumn extends ColumnSchemaBase<TInputScope, ImplicitAllowedTypes>>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, _cellSchema: TCell, columnSchema: TColumn): ReturnType<typeof createTableInternal<TInputScope, TCell, TColumn>>;
    /**
     * Factory for creating new table schema.
     * @internal
     */
    function createTable<const TInputScope extends string | undefined, const TCell extends ImplicitAllowedTypes, const TColumn extends ColumnSchemaBase<TInputScope, ImplicitAllowedTypes>, const TRow extends RowSchemaBase<TInputScope, TCell, ImplicitAllowedTypes>>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, _cellSchema: TCell, columnSchema: TColumn, rowSchema: TRow): ReturnType<typeof createTableInternal<TInputScope, TCell, TColumn, TRow>>;
    /**
     * Factory for creating new table schema.
     * @system @internal
     */
    function createTableInternal<const TInputScope extends string | undefined, const TCell extends ImplicitAllowedTypes, const TColumn extends ColumnSchemaBase<TInputScope, ImplicitAllowedTypes> = ColumnSchemaBase<TInputScope, ImplicitAllowedTypes>, const TRow extends RowSchemaBase<TInputScope, TCell, ImplicitAllowedTypes> = RowSchemaBase<TInputScope, TCell, ImplicitAllowedTypes>>(inputSchemaFactory: SchemaFactoryAlpha<TInputScope>, _cellSchema: TCell, columnSchema: TColumn, rowSchema: TRow): TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table">, NodeKind.Object, TreeNode & ITable<TCell, TColumn, TRow> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table">, NodeKind, unknown>, object & {
        readonly rows: (import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, true, TRow, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>> | undefined) => TreeArrayNode<TRow, [TRow] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TRow> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TRow] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TRow> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, true, TRow, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>> | undefined) => TreeArrayNode<TRow, [TRow] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TRow> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TRow] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TRow> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, unknown>)> | undefined) & import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, true, TRow, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>> | undefined) => TreeArrayNode<TRow, [TRow] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TRow> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TRow] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TRow> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, true, TRow, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>> | undefined) => TreeArrayNode<TRow, [TRow] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TRow> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TRow] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TRow> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, unknown>)>;
        readonly columns: (import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, true, TColumn, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>> | undefined) => TreeArrayNode<TColumn, [TColumn] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TColumn> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TColumn] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TColumn> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, true, TColumn, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>> | undefined) => TreeArrayNode<TColumn, [TColumn] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TColumn> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TColumn] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TColumn> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, unknown>)> | undefined) & import("./simple-tree/schemaTypes.js").InsertableTypedNode<import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, true, TColumn, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>> | undefined) => TreeArrayNode<TColumn, [TColumn] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TColumn> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TColumn] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TColumn> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, unknown>), import("./simple-tree/index.js").TreeNodeSchemaCore<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, true, TColumn, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>>, unknown> & (new (data?: import("./simple-tree/index.js").InternalTreeNode | Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>> | undefined) => TreeArrayNode<TColumn, [TColumn] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TColumn> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TColumn] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TColumn> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, unknown>)>;
    }, true, {
        readonly rows: TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, TreeArrayNode<TRow, [TRow] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TRow> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TRow] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TRow> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.rows">, NodeKind.Array, unknown>, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TRow>>, true, TRow, undefined>;
        readonly columns: TreeNodeSchemaClass<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, TreeArrayNode<TColumn, [TColumn] extends [ImplicitAllowedTypes] ? TreeNodeFromImplicitAllowedTypes<TColumn> : TreeNodeFromImplicitAllowedTypes<ImplicitAllowedTypes>, [TColumn] extends [ImplicitAllowedTypes] ? InsertableTreeNodeFromImplicitAllowedTypes<TColumn> : never, import("./simple-tree/arrayNode.js").ReadonlyArrayNode<TreeNode | import("./simple-tree/schemaTypes.js").TreeLeafValue>> & WithType<ScopedSchemaName<ScopedSchemaName<TInputScope, "table">, "Table.columns">, NodeKind.Array, unknown>, Iterable<InsertableTreeNodeFromImplicitAllowedTypes<TColumn>>, true, TColumn, undefined>;
    }>;
    /**
     * Base row schema type.
     * @sealed @system @internal
     */
    type TableSchemaBase<TScope extends string | undefined, TCell extends ImplicitAllowedTypes, TColumn extends ColumnSchemaBase<TScope, ImplicitAllowedTypes> = ColumnSchemaBase<TScope, ImplicitAllowedTypes>, TRow extends RowSchemaBase<TScope, TCell, ImplicitAllowedTypes> = RowSchemaBase<TScope, TCell, ImplicitAllowedTypes>> = ReturnType<typeof createTable<TScope, TCell, TColumn, TRow>>;
}
//# sourceMappingURL=tableSchema.d.ts.map