/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { SharedObject, createSharedObjectKind, } from "@fluidframework/shared-object-base/internal";
import { SharedTreeFactoryType, SharedTreeAttributes } from "./sharedTreeAttributes.js";
import { Breakable } from "./util/index.js";
import { SharedTreeKernel } from "./shared-tree/index.js";
import { UsageError } from "@fluidframework/telemetry-utils/internal";
import { fail } from "@fluidframework/core-utils/internal";
/**
 * Shared object wrapping {@link SharedTreeKernel}.
 */
class SharedTreeImpl extends SharedObject {
    constructor(id, runtime, attributes, optionsParam, telemetryContextPrefix = "fluid_sharedTree_") {
        super(id, runtime, attributes, telemetryContextPrefix);
        this.breaker = new Breakable("Shared Tree");
        if (runtime.idCompressor === undefined) {
            throw new UsageError("IdCompressor must be enabled to use SharedTree");
        }
        this.kernel = new SharedTreeKernel(this.breaker, this, this.serializer, (content, localOpMetadata) => this.submitLocalMessage(content, localOpMetadata), () => this.deltaManager.lastSequenceNumber, this.logger, runtime.idCompressor, optionsParam);
    }
    summarizeCore(serializer, telemetryContext, incrementalSummaryContext) {
        return this.kernel.summarizeCore(serializer, telemetryContext, incrementalSummaryContext);
    }
    processCore(message, local, localOpMetadata) {
        fail(0xb75 /* processCore should not be called on SharedTree */);
    }
    processMessagesCore(messagesCollection) {
        this.kernel.processMessagesCore(messagesCollection);
    }
    onDisconnect() {
        this.kernel.onDisconnect();
    }
    exportVerbose() {
        return this.kernel.exportVerbose();
    }
    exportSimpleSchema() {
        return this.kernel.exportSimpleSchema();
    }
    contentSnapshot() {
        return this.kernel.contentSnapshot();
    }
    viewWith(config) {
        return this.kernel.viewWith(config);
    }
    async loadCore(services) {
        await this.kernel.loadCore(services);
    }
    didAttach() {
        this.kernel.didAttach();
    }
    applyStashedOp(...args) {
        this.kernel.applyStashedOp(...args);
    }
    reSubmitCore(...args) {
        this.kernel.reSubmitCore(...args);
    }
}
/**
 * A channel factory that creates an {@link ITree}.
 */
class TreeFactory {
    constructor(options) {
        this.options = options;
        this.type = SharedTreeFactoryType;
        this.attributes = SharedTreeAttributes;
    }
    async load(runtime, id, services, channelAttributes) {
        const tree = new SharedTreeImpl(id, runtime, channelAttributes, this.options);
        await tree.load(services);
        return tree;
    }
    create(runtime, id) {
        const tree = new SharedTreeImpl(id, runtime, this.attributes, this.options);
        tree.initializeLocal();
        return tree;
    }
}
TreeFactory.Type = SharedTreeFactoryType;
/**
 * SharedTree is a hierarchical data structure for collaboratively editing strongly typed JSON-like trees
 * of objects, arrays, and other data types.
 * @legacy
 * @alpha
 */
export const SharedTree = configuredSharedTree({});
/**
 * {@link SharedTree} but allowing a non-default configuration.
 * @remarks
 * This is useful for debugging and testing to opt into extra validation or see if opting out of some optimizations fixes an issue.
 * @example
 * ```typescript
 * import {
 * 	ForestType,
 * 	TreeCompressionStrategy,
 * 	configuredSharedTree,
 * 	typeboxValidator,
 * 	// eslint-disable-next-line import/no-internal-modules
 * } from "@fluidframework/tree/internal";
 * const SharedTree = configuredSharedTree({
 * 	forest: ForestType.Reference,
 * 	jsonValidator: typeboxValidator,
 * 	treeEncodeType: TreeCompressionStrategy.Uncompressed,
 * });
 * ```
 * @privateRemarks
 * This should be legacy, but has to be internal due to limitations of API tagging preventing it from being both alpha and alpha+legacy.
 * TODO:
 * Expose Ajv validator for better error message quality somehow.
 * Maybe as part of a test utils or dev-tool package?
 * @internal
 */
export function configuredSharedTree(options) {
    class ConfiguredFactory extends TreeFactory {
        constructor() {
            super(options);
        }
    }
    return createSharedObjectKind(ConfiguredFactory);
}
//# sourceMappingURL=treeFactory.js.map